/**
 *@name geoxml3.js: parses KML/XML files for markers and overlays
 * @version 1.3.3-GoogleMaps Api V3
 * @copyright (c) 2010 B.Ostermann
 * @author Bernd Ostermann
 * @contact ostermann.bernd@arcor.de
 * <p>This script is based on geoXML3.js
 *  http://code.google.com/p/geoxml3/
 *
 *  <p>Modifications of the original geoxml3:
 *  <br>The original fetches xml files by applying
 *  XMLHttpRequest() or ActiveXObject("MSXML2.XMLHTTP.3.0") for IE.
 *  <br>Some browsers as Opera deny access to xml files by using XMLHttpRequest().
 *  <br>The intention of this javascript application is, that it can be launched form
 *  a web server or from the local file system.
 *  <br>Therfore the xml files were converted to strings and stored as objects in
 *  javascript files.
 *  <br>The string with the xml document is then parsed by
 *  <br> window.DOMParser or by ActiveXObject("Microsoft.XMLDOM") for IE.
 */
/**
 *Extends the global String with a method to remove leading and trailing
 *whitespaces.
*/
if (!String.prototype.trim) {
    String.prototype.trim = function () {
        return this.replace(/^\s+|\s+$/g, '');
    };
}

/**
 *Constructor for the root KML parser object
 *@param options parsing options
 *<ul>
 * <li>{ map: !=null: markers and overlays are rendered on the map;=null: no rendering
 * <li>zoom: =true: the map is centered and zoomed according the positons of the markers and overlays
 * <li>processStyles: =true: parsing for tags <Style>
 * <li>createMarker: function(placemark, countryCode) user defined callback function
 * for creating markers. Returns a google.map.Marker
 * <li>createOverlay: function(groundOverlay, doc) user defined callback
 * <li>singleInfoWindow: = true: clicking a marker will close an open infoWindow
 * <li>infoWindowOptions: google.map.InfoWindow options
 *  </ul>
 */

function GeoXML3(options) {
    // Private variables
    var parserOptions = combineOptions(options, {
        singleInfoWindow: false,
        processStyles: true,
        zoom: true
    });
    var lastMarker;
    var xmlParser;
    if (window.DOMParser){
        xmlParser=new XmlDomParser(); //all browsers except IE
    }else{
        xmlParser=new XmlIeDomParser(); //Internet Explorer
    }
    // public methods
    /**
     *Parses the xml documents of xmlObjects and returns an array of docs.
     *<br>If a parsing error occured an error message is issued.
     *@param xmlObjects {countryCode:2 uppercase letters,cngXml: xml document as a string}
     *@return docSet array of doc with
     *<br> doc= {countryCode,styles,markers,overlays};
     * markers and overlays: only, if option map was set.
     */
    this.parse = function (xmlObjects) {
        var docSet=null;
        lastMarker=null;
        docSet=new Array();
        // Process one or more XML/KML documents
        var nDocs=xmlObjects.length;  //number of docs to parse
        var xmlDoc;
        var doc;
        var xmlObject;
        var countryCode;
        var bounds=null;        //overall bounds of markers and overlays of all docs
        if (parserOptions.zoom && parserOptions.map){
            bounds=new google.maps.LatLngBounds();
        }

        for (var i = 0; i < nDocs; i++) {
            xmlObject=xmlObjects[i];
            countryCode=xmlObject.countryCode;
            try{
                xmlDoc=xmlParser.getXmlDoc(xmlObject.gasXml);
                doc=render(xmlDoc, countryCode);
                docSet.push(doc);
                if (bounds){
                    bounds.union(doc.bounds);
                }
            }catch(err){
                alert(err + " " + countryCode);
            }
        }
        //all xmlObjects are parsed
        if (bounds) {
            parserOptions.map.fitBounds(bounds);
        }
        return docSet;
    };
    /**
     * A public function for creating a googlemaps.Marker
     * @param placemark {name, description, icon, shadow}
     * @param htmlImgFlag a html tag used as prefix in the text of the InfoWindow
     * @return google.map.Marker
     */
    this.createMarker2=function(placemark, htmlImgFlag) {
        return createMarker(placemark, htmlImgFlag);
    };
    /**
     *If parserOptions.map is defined, parsing a document stores markers or overlays
     *in document.markers or documnet.overlays). After parsing showDocument will
     * make them visible..
     *@param doc {countryCode,styles,markers,overlays}
     *<br>markers and overlays: only, if option map was set.
     */
    this.showDocument = function (doc) {
        var map=parserOptions.map;
        if (map){
            var marker;
            var i;
            for (i = 0; i < doc.markers.length; i++) {
                marker=doc.markers[i];
                marker.setMap(parserOptions.map);
                marker.setVisible(true);
            }
            for (i = 0; i < doc.overlays.length; i++) {
                doc.overlays[i].setOpacity(doc.overlays[i].percentOpacity_);
            }
        }
    };
    /**
     * Hides the map objects associated with a document.
     * @param doc  {countryCode,styles,markers,overlays}
     * <br>markers and overlays: only, if option map was set.
     */
    this.hideDocument = function (doc) {
        var i;
        for (i = 0; i < doc.markers.length; i++) {
            doc.markers[i].setVisible(false);
        }
        for (i = 0; i < doc.overlays.length; i++) {
            doc.overlays[i].setOpacity(0);
        }
    };
    //private classes
    /**
     *The parser for all browsers except IE.
     *<br>Defines the public function getXmlDoc which gets the parsed XML document as a DOM object.
     *@return xmlDoc the parsed document as a DOM object
     */
    function XmlDomParser(){
        var parser=new DOMParser();
        this.getXmlDoc=function(xmlString){
            var xmlDoc=parser.parseFromString(xmlString,"text/xml");
            if (xmlDoc.documentElement.nodeName == "parsererror"){
                var msg="Error while parsing XML document";
                throw(msg);
            }
            return xmlDoc;
        };
    }
    /**
     *The parser for IE.
     *<br>Defines the public function getXmlDoc which gets the parsed XML document as a DOM object.
     *@return xmlDoc the parsed document as a DOM object
     */
    function XmlIeDomParser(){
        var xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
        xmlDoc.async="false";
        this.getXmlDoc=function(xmlString){
            xmlDoc.loadXML(xmlString);
            var errorObj=xmlDoc.parseError;
            var errorCode=errorObj.errorCode;
            if (errorCode!=0){
                var reason=errorObj.reason + "\n Xml Document ";
                throw(reason);
            }
            return xmlDoc;
        };
    }
    //private methods
    /**
     * Extracts markers and overlays from the DOM object and creates map objects.
     * <br>If the option createMarker or createOverlay was set, the user defined functions are used
     * for creation, otherwise the built-in functions.
     * <br>If the option map !=null was set, markers and/or overlays are rendered on the map.
     * <br>If the option zoom=true and map !=null was set, the map will fit to the bounds
     * of the rendered objects.
     *@param xmlDoc the parsed XML file
     *@param countryCode 2 uppercase letters definig the country like UK, FR, IT
     **@return doc= {countryCode,styles,markers,overlays}
     *<br>markers and overlay are filled only if option map was set.
     */
    function render(xmlDoc, countryCode) {  
        var doc= {
            countryCode: countryCode,
            styles:new Array(),
            detailBases:new Array(),    //base urls of Details
            //placemarks: new Array(),
            markers:new Array(),
            overlays:new Array()
        };
        if (parserOptions.zoom && parserOptions.map)
            doc.bounds = new google.maps.LatLngBounds();
        // Parse styles
        var styleID, nodes, i;
        var styleNodes = xmlDoc.getElementsByTagName('Style');
        for (i = 0; i < styleNodes.length; i++) {
            styleID   = styleNodes[i].getAttribute('id');
            nodes = styleNodes[i].getElementsByTagName('Icon');
            if (!!nodes.length) {
                doc.styles['#' + styleID] = {
                    href: nodeValue(nodes[0].getElementsByTagName('href')[0])
                };
            }
        }
        if (!!parserOptions.processStyles || !parserOptions.createMarker) {
            // Convert parsed styles into GMaps equivalents
            processStyles(doc);
        }
        //Parse DetailBase
        if (parserOptions.processDetails){
            var baseurl;
            var id;
            var detailBaseNodes = xmlDoc.getElementsByTagName('DetailBase');
            for (i = 0; i < detailBaseNodes.length; i++) {
                id= detailBaseNodes[i].getAttribute('id');
                baseurl=nodeValue(detailBaseNodes[i]);
                doc.detailBases['#' + id]=baseurl;
            }
        }
        // Parse placemarks
        var placemark, node, coords, path;
        var placemarkNodes = xmlDoc.getElementsByTagName('Placemark');
        //alert("render; Anzahl placemarks= "+ placemarkNodes.length);
        var marker;
        var url1,url;
        var urlNodes;
        for (i = 0; i < placemarkNodes.length; i++) {
            // Init the placemark object
            node = placemarkNodes[i];
            if (parserOptions.processDetails){
                urlNodes=node.getElementsByTagName('DetailUrl1');
                if (urlNodes.length>0){
                    id=nodeValue(urlNodes[0]);
                    url1=doc.detailBases[id];
                    url=url1 + nodeValue(node.getElementsByTagName('DetailUrl2')[0]);
                }else{
                    url=null;
                }
            }else{
                url=null;
            }
            placemark = {
                name:  nodeValue(node.getElementsByTagName('name')[0]),
                description: nodeValue(node.getElementsByTagName('description')[0]),
                styleUrl: nodeValue(node.getElementsByTagName('styleUrl')[0]),
                detailUrl: url
            };
            placemark.style = doc.styles[placemark.styleUrl] || {};
            if (/^https?:\/\//.test(placemark.description)) {
                placemark.description = '<a href="' + placemark.description + '">' + placemark.description + '</a>';
            }
            
            // Extract the coordinates
            coords = nodeValue(node.getElementsByTagName('coordinates')[0]).trim();
            coords = coords.replace(/\s+/g, ' ').replace(/, /g, ',');
            path = coords.split(' ');
            // What sort of placemark?
            if (path.length === 1) {
                // Polygons/lines not supported in v3, so only plot markers
                coords = path[0].split(',');
                placemark.point = {
                    lat: parseFloat(coords[1]),
                    lng: parseFloat(coords[0]),
                    alt: parseFloat(coords[2])
                };
                if (!!doc.bounds) {
                    doc.bounds.extend(new google.maps.LatLng(placemark.point.lat, placemark.point.lng));
                }
                // Call the appropriate function to create the marker
                if (parserOptions.createMarker) {
                    //user define function
                    marker=parserOptions.createMarker(placemark, countryCode);
                } else {
                    //built-in function
                    createMarker(placemark, '');
                }
                if (parserOptions.map){
                    marker.setMap(parserOptions.map);
                    doc.markers.push(marker);
                }
            }
        }
        // Parse ground overlays
        var groundOverlay, color, transparency;
        var groundNodes = xmlDoc.getElementsByTagName('GroundOverlay');
        var overlay;
        for (i = 0; i < groundNodes.length; i++) {
            node = groundNodes[i];
            // Init the ground overlay object
            groundOverlay = {
                name: nodeValue(node.getElementsByTagName('name')[0]),
                description: nodeValue(node.getElementsByTagName('description')[0]),
                icon: {
                    href: nodeValue(node.getElementsByTagName('href')[0])
                },
                latLonBox: {
                    north: parseFloat(nodeValue(node.getElementsByTagName('north')[0])),
                    east:  parseFloat(nodeValue(node.getElementsByTagName('east')[0])),
                    south: parseFloat(nodeValue(node.getElementsByTagName('south')[0])),
                    west:  parseFloat(nodeValue(node.getElementsByTagName('west')[0]))
                }
            };
            if (!!doc.bounds) {
                doc.bounds.union(new google.maps.LatLngBounds(
                    new google.maps.LatLng(groundOverlay.latLonBox.south, groundOverlay.latLonBox.west),
                    new google.maps.LatLng(groundOverlay.latLonBox.north, groundOverlay.latLonBox.east)
                    ));
            }
            // Opacity is encoded in the color node
            color = nodeValue(node.getElementsByTagName('color')[0]);
            if ((color !== '') && (color.length == 8)) {
                transparency = parseInt(color.substring(0, 2), 16);
                groundOverlay.opacity = Math.round((255 - transparency) / 2.55);
            } else {
                groundOverlay.opacity = 100;
            }
            // Call the appropriate function to create the overlay
            if (!!parserOptions.createOverlay) {
                //user defined function
                overlay=parserOptions.createOverlay(groundOverlay, doc);
                if (parserOptions.map){
                    overlay.setMap(parserOptions.map);
                    doc.overlays.push(overlay);
                }
            } else {
                //built-in function
                createOverlay(groundOverlay, doc);
            }
        }
        return doc;
    }
    /**
     * Defines for each styleID the corresponding MarkerImage and shadow.
     * @param doc {countryCode: countryCode,styles:new Array(),placemarks: new Array(),
     *  markers:new Array(),overlays:new Array()};
     */
    function processStyles(doc) {
        var stdRegEx = /\/(red|blue|green|yellow|lightblue|purple|pink|orange)(-dot)?\.png/;
        for (var styleID in doc.styles) {
            if (!!doc.styles[styleID].href) {
                // Init the style object with a standard KML icon
                doc.styles[styleID].icon =  new google.maps.MarkerImage(
                    doc.styles[styleID].href,
                    new google.maps.Size(32, 32),
                    new google.maps.Point(0, 0),
                    new google.maps.Point(0, 32));
                // Look for a predictable shadow
                if (stdRegEx.test(doc.styles[styleID].href)) {
                    // A standard google.maps-style marker icon
                    doc.styles[styleID].shadow = new google.maps.MarkerImage(
                        'http://maps.google.com/mapfiles/ms/micons/msmarker.shadow.png',
                        new google.maps.Size(50, 32),
                        new google.maps.Point(0, 0),
                        new google.maps.Point(0, 32));
                } else if (doc.styles[styleID].href.indexOf('pushpin',0) > -1) {
                    // Pushpin marker icon
                    doc.styles[styleID].shadow = new google.maps.MarkerImage(
                        'http://maps.google.com/mapfiles/ms/micons/pushpin_shadow.png',
                        new google.maps.Size(50, 32) ,
                        new google.maps.Point(0, 0),
                        new google.maps.Point(0,32));
                } else {
                    // without shadow
                    doc.styles[styleID].shadow=null;
                    /*
                    i=doc.styles[styleID].href.indexOf('pushpin',0);
                    doc.styles[styleID].shadow = new google.maps.MarkerImage(
                        doc.styles[styleID].href.replace('.png', '.shadow.png'),
                        new google.maps.Size(50, 32),
                        new google.maps.Point(0, 0),
                        new google.maps.Point(0, 32));
                    */
                }
            }
        }
    }

    /**
     * Built-in function for creating a google.maps.Marker, if the
     * option createMarker is not set. The marker is not rendered on the amp.
     * @param placemark {name, description, icon, shadow}
     * @param htmlFlagImg a html tag used as prefix in the text of the InfoWindow
     * @return google.map.Marker
     */
    function createMarker(placemark, htmlFlagImg) {
        // create a Marker to the map from a placemark KML object
        // Load basic marker properties
        var markerOptions = combineOptions(parserOptions.markerOptions, {
            //map:      null,
            position: new google.maps.LatLng(placemark.point.lat, placemark.point.lng),
            title:    placemark.name,
            zIndex:   Math.round(-placemark.point.lat * 100000),
            icon:     placemark.style.icon,
            shadow:   placemark.style.shadow
        });
        // Create the marker on the map
        var marker = new google.maps.Marker(markerOptions);
        // Set up and create the infowindow
        var detailUrl=placemark.detailUrl;
        var content='<div>' + htmlFlagImg + '&nbsp;<b>' + placemark.name +
        '</b><br/>' + placemark.description;
        if (detailUrl!=null){
            content+='<p><a href=' + placemark.detailUrl + ' target="_blank">Details</a></p></div>';
        }else{
            content+='</div>';
        }
        var infoWindowOptions = combineOptions(parserOptions.infoWindowOptions, {
            content:content ,
            pixelOffset: new google.maps.Size(0, 2)
        });
        marker.infoWindow = new google.maps.InfoWindow(infoWindowOptions);
        // Infowindow-opening event handler
        google.maps.event.addListener(marker, 'click', function() {
            if (!!parserOptions.singleInfoWindow) {
                if (!!lastMarker && !!lastMarker.infoWindow) {
                    lastMarker.infoWindow.close();
                }
                lastMarker = this;
            }
            this.infoWindow.open(this.map, this);
        });
        return marker;
    }
  
    function createOverlay(groundOverlay, doc) {
        // Add a ProjectedOverlay to the map from a groundOverlay KML object
        if (!window.ProjectedOverlay) {
            throw 'geoXML3 error: ProjectedOverlay not found while rendering GroundOverlay from KML';
        }
        var bounds = new google.maps.LatLngBounds(
            new google.maps.LatLng(groundOverlay.latLonBox.south, groundOverlay.latLonBox.west),
            new google.maps.LatLng(groundOverlay.latLonBox.north, groundOverlay.latLonBox.east)
            );
        var overlayOptions = combineOptions(parserOptions.overlayOptions, {
            percentOpacity: groundOverlay.opacity
        });
        var overlay = new ProjectedOverlay(parserOptions.map, groundOverlay.icon.href, bounds, overlayOptions);
    
        if (!!doc && parserOptions.map) {
            //doc.overlays = doc.overlays || [];
            doc.overlays.push(overlay);
        }
        return;
    }

    /**
     *Log a message to the debugging console, if one exists.
     *@param msg message text to log.
     */
    function log(msg) {
        if (!!window.console) {
            console.log(msg);
        }
    }

    /**
     *Combines two options objects, a set of default values and a set of override values
     *@param overrides the overriding options
     *@param defaults the defualt options
     *@return resulting optins
     */
    function combineOptions(overrides, defaults) {
        var result = {};
        if (!!overrides) {
            for (var prop in overrides) {
                if (overrides.hasOwnProperty(prop)) {
                    result[prop] = overrides[prop];
                }
            }
        }
        if (!!defaults) {
            for (prop in defaults) {
                if (defaults.hasOwnProperty(prop) && (result[prop] === undefined)) {
                    result[prop] = defaults[prop];
                }
            }
        }
        return result;
    }
  
    /**
     * Extracts the text value of a DOM node, with leading and trailing whitespace trimmed
     * @param node a DOM node
     * @return the text of the DOM node
     */
    function nodeValue(node) {
        if (!node) {
            return '';
        } else {
            var txt=(node.innerText || node.text || node.textContent);
            return txt.trim();
        }
    }
}//end of object GeoXML3
