/**
 *@name routeClasses.js
 * @version 1.3.8 with GoogleMpas V3
 * @copyright (c) 2010 B.Ostermann
 * @author Bernd Ostermann
 * @contact ostermann.bernd@arcor.de
 * <br>Classes:
 * <br>MapDistanceControl
 * <br>RouteSections
 * <br>RefuelStop
 * <br>RefuelStops
 */
//Global variables:
var earthRadius=6378.137;        //radius of earth in kms
var toRadians=Math.PI/180.0;    //factor to convert degrees in radians
var toDegrees=1.0/toRadians;    //factor to convert radians in degrees
// Extend the global String with a method to remove leading and trailing whitespace
if (!String.prototype.trimSpec) {
    String.prototype.trimSpec = function () {
        return this.replace(/^\s+|\s+$/g, '');
    };
}
/**
 * Extends the global Array with a method to check the Identity with another array
 * @param array the array to be checked
 */
Array.prototype.isIdentical=function(array){
    var rv;
    var lengthArray=array.length;
    var thisLength=this.length;
    if (lengthArray== thisLength){
        rv=true;
        for (var i=0; i< thisLength; i++){
            if (this[i]!= array[i]){
                rv=false;
                break;
            }
        }
    }else{
        rv=false;
    }
    return rv;
};

/**
 *Class MapDistanceControl: a text field with the distance of the
 *mouse cursor to the route
*/
function MapDistanceControl() {
    var container = document.createElement("div");
    container.style.fontFamily='Arial';
    container.style.fontSize='XX-Small';
    var ptr = document.createElement("INPUT");
    ptr.style.marginTop="5px";
    ptr.style.width = "110px";
    ptr.style.height = "15px";
    ptr.type = "Text";
    ptr.readOnly = true;
    ptr.id = "distPtr";
    ptr.title="distance of mouse to route";
    container.appendChild(ptr);
    return container;
}
/**
  *Filtering POIs along the route and around centers of refueling stops.
  *<br>Principles:
  *<br>The method geoDistanceToPolyMtrs(poly, point) of ./lib/BdccGeo.js calculates
  * the shortest distance to the route poly. This is done by
  * determining the minimum distance of all verteces of poly to the point.
  * The number of verteces of a route may be some thousends. The number
  * of the POIs CNG stations is about 2000. So the number of distance calcualtions
  * is in the order of some millions. The time for filtering the
  * POIs may be some minutes.
  * <br>Reducing the number of distance calculations:
  * <br>1. step:
  * <br>In distance calculation the overview_path of the route is used, not its
  * polyline. This reduces the number of verteces by a factor of apprx. 10.
  *
  * <br>2. step:
  * <br>The reduced route polyline is covered by a sequence of overlapping sections with a section
  * polyline. The length of a section and its polyline is dependent on the route length and
  * the maximum distance, the POIs may be near the route.
  * <br>Filtering POIs nearby the route:
  * <br>First is tested, if the POI is within one of the sections.
  * <br>If the POI is within a section, the distance to the section
  * polyline is determined.
  * <br>If this distance is less or equal maxDistance, the POI is
  * selected.
  * <br>Filtering POIs nearby centers of refueling stations:
  * <br>The centers of refueling stations along the route in a distance
  * of (refueling-range - Max(maxDistance,refueling-range/10) are determined.
  * <br>If a POI is within one of the circles with radius Max(Sqrt(2)*maxDistance, range/10)
  * around center, then the section filter is applied (s.above).
  */

/**
  *Class RouteSections:
  *<br>generates
  *overlapping rectangular sections, which cover the route.
  *<br>Each section consists of a rectangle, its bounds, the polyline
  *and the length from route start to the section end.
  *<br>It provides the external accessible methods filterPoi and clearBoxes
  *@param map the actual map
  *@param routeObj {route, route length in kms}
  *@param maxDistance the maximum distance of a POI from the route in kms
  */

function RouteSections(map, routeObj, maxDistance){
    this.routePolyline=routeObj.routePolyline;
    var _map=map;
    var _routeLength=routeObj.routeLength;
    //private members
    var _maxDistance=maxDistance;   //maximum distance from the route
    var _limitMaxDistance=50.0;     //upper limit of amxDistance
    var _polygons=new Array();      //array of rectangles covering the route
    var _boundsOfSections=new Array();  //bounds of rectangles
    var _routeStartTosectionEnds=new Array(); //distances from route start to section ends
    var _polylines = new Array();   //array of polylines within a section
    var routeBounds=routeObj.bounds;    //bounds of the route
    //bounds extended by max Distance
    var _overallBounds=getRectangle(routeBounds.getSouthWest(), routeBounds.getNorthEast(), maxDistance).getBounds();
    //construct a sequence of sections surrounding the route in a distance of maxDistance
    getSectionsOfRoute(this.routePolyline,_routeLength,_maxDistance);

    //privileged public method, which can access private members

    /**
      *Checks, if the point 'latLng' is within one of the sections.
      *<br>If so, determines the distance from the route.
      *<br>If the distance from the route is less or equal to this.maxDistance,
      * object {distanceFromRoute, distanceFromStart} is determined.
      * @param latLng latitude and longitude of the point
      * @return object !=null: {distanceFromRoute, distanceOnRoute}
      * <br>distanceFromRoute: airline distance from the route in kms
      * <br>distanceOnRoute: distance on the roue from route start in kms
      * <br> object=null: the point is farer away from the route than maxDistance
      */
    this.filterPoi=function(latLng){
        var rv=null;
        if (_overallBounds.contains(latLng)){
            var sectionIndex=isInSectionsBounds(latLng);
            if (sectionIndex>-1){
                var sectionPolyline=_polylines[sectionIndex];
                var objDist=getProximity(sectionPolyline, latLng); //in kms
                var dist=objDist.distance;
                if (dist<=_maxDistance){
                    var distRefpoint=getRouteLengthOfRefPoint(sectionIndex,objDist.routeIndex);
                    rv={
                        'distanceFromRoute': Math.round(dist*1000)/1000.0,
                        'distanceOnRoute':Math.round(distRefpoint*1000)/1000.0
                    };
                }
            }
        }
        return rv;
    };
    /**
     *Removes all existing polygons and polylines from the map.
     *<br>class variable: array _polygons with all polygons
     */
    this.clearBoxes=function(){
        if (_polygons){
            var n= _polygons.length;
            for (var i=0; i<n; i++){
                _polygons[i].setMap(null);
                _polylines[i].setMap(null);
            }
        }
    };
    this.getBoundsOfSections=function(){
        return _boundsOfSections;
    };
    //private methods only accessable by priviliged or private methods
    /**
      *Constructs overlapping sections, which covers the polyline.
      *<br>Each section has following properties:
      *<br>rectangular polygon
      *<br> the bounds of the polygon
      *<br> section polyline
      *<br> the distance from route start to section end
      *<br>These properties are stored in the internal arrays
      *<br>_polygons, _polylines, _boundsOfSections, routeStartTosectionEnds
      *<br>number of sections:
      *<br>if route distance is less or equal 50 kms: 1 section
      *<br>otherwise: minimum 100 and (route length)/maxDistance*4
      *<p>Reduction of polyline 'routePloly':
      *<br>The number of verteces of the polyline is critical for the
      *response time. Not all NV verteces are used, but each n th vertex.
      *<br>d0=mean distance of subsequent verteces
      *<br>deviation of reduced and complete polyline= (n-1)*d0
      *<br>maxDev= maximum deviation
      *<br>n is determined by: (n-1)*do = maxDev
      *<br>maxDev is defined as:
      *<br> maxDistance &lt;= 10: maxDistance*0.08
      *<br>10 &lt;= maxDistance lt;_limitMaxDistance:
      *<br>maxDev linearly decreasing from maxDistance*0.08 to maxDistance*0.02
      *@param routePoly polyline of the route
      *@param distRoute distance of the route in kms
      *@param maxDistance maximum distance from the route in kms
      */
    function getSectionsOfRoute(routePoly, distRoute, maxDistance){
        var path=routePoly.getPath();
        var nVerteces=path.length;
        //construct sections of the route
        var distSection;
        var maxSections=100;
        var deltaVertexIndex;
        var d0=distRoute/nVerteces; //mean distance of verteces in kms
        if(distRoute <= 50.0){
            //only 1 section
            distSection=distRoute;
            deltaVertexIndex=1;
        }else{
            var maxDev;
            if (maxDistance >10.0){
                var max=Math.min(maxDistance, _limitMaxDistance);
                maxDev=max*(0.10 - (max-10.0)*0.08/(_limitMaxDistance-10.0));
            }else{
                maxDev=maxDistance*0.10;
            }
            deltaVertexIndex=Math.floor(maxDev/d0+1);
            if (deltaVertexIndex< 1){
                deltaVertexIndex=1;
            }
            nReducedPolyline=nVerteces/deltaVertexIndex;
            if (distRoute/maxSections < maxDistance*4){
                distSection=maxDistance*4;
            }else{
                distSection=distRoute/maxSections;
            }
        }
        //alert("no verteces polyline= " + nVerteces + ", no verteces reduced: " + nReducedPolyline + ", deltaIndex= " +
        //deltaVertexIndex + ", error= " + error + ", d0= " + d0);
        var sectionLength;      //length of the section polyline without extension
        var overallLength=0.0;  //length of all preceding sections
        var iv=0;
        var vertex1;
        var vertex2=path.getAt(0);
        var vertex;
        var polyline;
        var dist;
        var ir;
        var color="yellow";
        var width=4;
        var noSection=0;
        var polyOpts;
        var latMin, latMax,lngMin, lngMax;
        var polyBounds;
        while(iv < (nVerteces-deltaVertexIndex)){
            noSection++;
            var points=new Array();
            sectionLength=0.0;
            vertex1=vertex2;    //vertex2: end point of last section polyline
            latMax=latMin=vertex1.lat();
            lngMax=lngMin=vertex1.lng();
            points.push(vertex1);
            //find first point on route with distance from vertex1 >=distSection
            while (iv < nVerteces-deltaVertexIndex) {
                iv+=deltaVertexIndex;
                vertex2=path.getAt(iv);
                points.push(vertex2);
                latMin=Math.min(latMin, vertex2.lat());
                latMax=Math.max(latMax, vertex2.lat());
                lngMin=Math.min(lngMin, vertex2.lng());
                lngMax=Math.max(lngMax, vertex2.lng());
                sectionLength+=geoDistance(vertex1,vertex2);
                if (sectionLength >= distSection){
                    break;
                }
                vertex1=vertex2;
            }
            var swLatLng=new google.maps.LatLng(latMin, lngMin);
            var neLatLng=new google.maps.LatLng(latMax, lngMax);
            var polygon=getRectangle(swLatLng, neLatLng, maxDistance);
            //extend polyline by maxDistance
            ir=iv;
            vertex1=vertex2;
            while (ir < nVerteces-deltaVertexIndex) {
                ir+=deltaVertexIndex;
                vertex=path.getAt(ir);
                points.push(vertex);
                dist=geoDistance(vertex1,vertex);
                if (dist >=maxDistance ){
                    break;
                }
            }
            if (color=="yellow"){
                color="red";
                width=8;
            }else{
                color="yellow";
                width=2;
            }
            polyOpts={
                path:points,
                strokeColor:color,
                strokeOpacity:0.4,
                strokeWeight:width
            };
            polyline=new google.maps.Polyline(polyOpts);
            //polyline.setMap(_map);
            _polygons.push(polygon);
            _polylines.push(polyline);
            polyBounds=polygon.getBounds();
            _boundsOfSections.push(polyBounds);
            _routeStartTosectionEnds.push(overallLength + sectionLength);
            overallLength+=sectionLength;
        //polygon.setMap(_map);
        }
    //alert("RouteSetions;number of sections= " + _polygons.length);
    }

    /**
      *Tests, if the point "latLng" is in one of the route sections.
      *@param latLng the point to be tested
      *@return index>-1: the point is within the section 'index'
      *<br> index=-1: the point is not within any section bounds
      */
    function isInSectionsBounds(latLng){
        var rv=-1;
        var nSections=_boundsOfSections.length;
        for(var i=0; i<nSections;i++){
            if(_boundsOfSections[i].contains(latLng)){
                rv=i;
                break;
            }
        }
        return rv;
    }
    /**
    * Gets the rectangle given by the south west and
    * north east points, extended by offsetKms.
    * <br>An extension of 1 km corresponds to a shift of angles:
    * <br> delta latitude=toDegrees/earthRadius
    * <br> delta longitude toDegrees/(earthRadius*Cos(latitude))
    * <br>note:
    * <br>the line element of a sphere is:
    * <br>ds^2=dr^2 + r^2*cos(lat)*dlng^2 + r2*dlat^2
    * <br>dlng=0 or dlat=0 results in delta latitude and delta longitude (s.above)
    * @param swLatLng south west point
    * @param neLatLng north eath point
    * @param offsetKms offset in kms
    * @return rectangle
    */
    function getRectangle(swLatLng, neLatLng, offsetKms){
        var offsetLat=offsetKms*toDegrees/earthRadius;
        var avLat=(swLatLng.lat()+ neLatLng.lat())*0.5*toRadians;
        var offsetLng=offsetLat/Math.cos(avLat);
        var sw=new google.maps.LatLng(swLatLng.lat()- offsetLat, swLatLng.lng()-offsetLng);
        var ne=new google.maps.LatLng(neLatLng.lat()+ offsetLat, neLatLng.lng()+ offsetLng);
        var opts={
            bounds:new google.maps.LatLngBounds(sw,ne),
            //map:_map,
            fillColor:"red",
            fillOpacity:0.1,
            strokeColor:"red",
            strokeOpacity:0.9,
            strokeWeight:1
        };
        var rect=new google.maps.Rectangle(opts);
        return rect;
    }
    /**
     *Gets the length on the route from route start to the reference point
     * on the route in kms.
     * <br>Reference point of a route external point: the point on the route with the
     * shortest distance to the external point.
     * @param sectionIndex index of the route section, which contains the reference point.
     * @param refPointIndex vertex index of the reference point on the section polyline.
     * @return length of the route from the route start to the reference point in kms.
     */

    function getRouteLengthOfRefPoint(sectionIndex,refPointIndex){
        var polyline=_polylines[sectionIndex];
        var path=polyline.getPath();
        var vertex1=path.getAt(0);
        var dist=0;
        var vertex;
        //length of the route within the actual section
        for (var i=1; i <=refPointIndex; i++){
            vertex=path.getAt(i);
            dist+=geoDistance(vertex1,vertex);  //distance in kms
            vertex1=vertex;
        }
        if (sectionIndex>0){
            dist+=_routeStartTosectionEnds[sectionIndex-1];
        }
        return Math.round(dist*1000)/1000.0;
    }
}

/**
  * Object RefuelStop:
  * @param center vertex of the center of refueling stop on the route polyline
  * @param vertexIndexOfCenter the vertex index on the route of the center
  * @param routeLengthToCenter the length on the route from route start to the center
  */
function RefuelStop(center,vertexIndexOfCenter,routeLengthToCenter){
    this.center=center;
    this.vertexIndexOfCenter=vertexIndexOfCenter;
    this.routeLengthToCenter=routeLengthToCenter;
}
/**
  * Combined class of RefuelStops and RouteSections.
  * <br>RefuelStops with centers of refuiling stations, which are 'cruiseRange' kms apart.
  * <br>Filtering POIs by public method filterPois:
  * <br>1. step: the POIs are selected, which are within a circle of
  * radius criuseRange/20 of one of the centers of a Refueling Stops.
  * <br>2. step: the POIs within one RefuelStop circle are selected, whose
  * distance from the route is less or equal maxDistance by method
  * RouteSections.filterPoi.
  * @param cruiseRange the distance to drive with a full tank in kms.
  * @param sharedInfoWin a InfoWindow for pop windows
  * @param map actual map
  * @param routeObj {route, route length in kms}
  * @param maxDistance maximum distance from the route in kms
  */
function RefuelStops(cruiseRange, sharedInfoWin,map,routeObj, maxDistance){
    //private members, only accessible by private or privileged methods
    var _routeSections=new RouteSections(map,routeObj, maxDistance);
    var _maxDistance=maxDistance;    //max distance of the POIs from the route
    var _routePoly=_routeSections.routePolyline; //polyline of the route
    var _map=map;
    var _centerMarkers=new Array();                 //markers of the centers of the refuiling stops
    var _cruiseRange=cruiseRange;                   //range with a full tank in kms
    var _sharedInfoWindow=sharedInfoWin;
    var _refuelStops=getRefuelStops(_routePoly, _maxDistance,cruiseRange);
    
    //privileged methods, which can access private methods and members
    
    this.filterPoi=function(latLng){
        var radius=Math.max(_maxDistance*1.41, _cruiseRange/10.0);
        var distances=null;
        var stopInd=isPointNearRefuelStop(latLng, radius);
        if (stopInd>-1){
            distances=_routeSections.filterPoi(latLng);
        }
        return distances;
    };
    /**
     *Removes all markers of refueling stops and boxes from the map.
     */
    this.clear=function(){
        _routeSections.clearBoxes();
        var n=_centerMarkers.length;
        for(var i=0; i< n; i++){
            _centerMarkers[i].setMap(null);
        }
    };
    //private methods
    /**
    *Creates a red or blue marker with a number index modulo 100.
    *<br>An event listener is defined, which pops up a window, after the amrker is
    *clicked.
    *@param point position of the marker as GLatLng
    *@param index the number of the marker
    *@param iconType =0: red, =1: blue
    *@param name name of the marker
    *@return marker object
    */
    function createNumberedMarker(point, index, iconType,name) {
        // Create an icon
        var i=index%100 +1;
        var url;
        if (iconType==0){
            //numbered red icon
            url="http://gmaps-samples.googlecode.com/svn/trunk/markers/red/marker" + i + ".png";
        }else{
            //blue numbered icon
            url="http://gmaps-samples.googlecode.com/svn/trunk/markers/blue/marker" + i + ".png";
        }
        var image = new google.maps.MarkerImage(url,
            new google.maps.Size(20, 26),
            new google.maps.Point(0,0),       //origin
            new google.maps.Point(10, 26));    //anchor
           
        url='http://maps.google.com/mapfiles/ms/micons/msmarker.shadow.png';
        var shadow = new google.maps.MarkerImage(url,
            // The shadow image is larger in the horizontal dimension
            // while the position and offset are the same as for the main image.
            new google.maps.Size(38, 26),
            new google.maps.Point(0,0),
            new google.maps.Point(10, 26));
        // Set up our MarkerOptions object
        var text=i +". " + name;
        var opts={
            //map: map,
            position:point,
            title:text,
            icon:image,
            shadow:shadow
        };
        var marker = new google.maps.Marker(opts);
        google.maps.event.addListener(marker, "click", function() {
            var content=text + "<br/>" + point.toUrlValue(6);
            _sharedInfoWindow.close();
            _sharedInfoWindow.setContent(content);
            _sharedInfoWindow.open(map,marker);
        });
        //marker.setMap(map);
        return marker;
    }
    /**
      *Gets refueling stops on the route polyline, which have a
      *distance of 'cruiseRange' - MAX('maxDistance','cruiseRange/10') from each other.
      *@param routePoly the polyline of a route
      *@param maxDistance maximum distance from the route in kms
      *@param cruiseRange the driving distance with a full tank in kms
      *@return array of RefuelStop
      */
    function getRefuelStops(routePoly, maxDistance,cruiseRange){
        var refuelStops=new Array();
        var marker;
        var vertex;
        var j=0;
        var firstIndex=0;
        var routeLength=0;
        //first refuel stop at route start
        vertex=routePoly.getPath().getAt(firstIndex);
        var refuelStop=new RefuelStop(vertex,firstIndex,routeLength);
        var d;
        var text;
        var markerDesc= new MarkerDesc();
        while(refuelStop){
            if (refuelStop){
                routeLength+=refuelStop.routeLengthToCenter;
                refuelStop.routeLengthCenter=routeLength;
                refuelStops.push(refuelStop);
                vertex=refuelStop.center;
                d=Math.round(routeLength*1000)/1000.0;
                text=markerDesc.refuelCenter + d + " km";
                marker=createNumberedMarker(vertex, j, 1, text);
                marker.setMap(_map);    //adds the marker to the map
                _centerMarkers.push(marker);
                firstIndex=refuelStop.vertexIndexOfCenter;
                j++;
            }
            refuelStop=getRefuelStop(routePoly, firstIndex, maxDistance, cruiseRange);
        }
        return refuelStops;
    }
    /**
      *Gets the next refuiling stop in a distance of MAX(maxDistFromRoute, cruiseRange/10.0).
      *@param routePolyline the polyline of a route
      *@param firstIndex the first vertex index on the route polyline to start
      *@param maxDistFromRoute the maximum istance from the oute in kms
      *@param cruiseRange the cruising range with a full tank in kms.
      *@return object RefuelStop (s. above)
      */
    function getRefuelStop(routePolyline, firstIndex, maxDistFromRoute, cruiseRange){
        var path=routePolyline.getPath();
        var vertex1=path.getAt(firstIndex);
        var vertex;
        var refuelStop=null;
        var n=path.length;
        //position of the next refuel stop
        var delta=Math.max(maxDistFromRoute, cruiseRange/10.0);
        var limit=cruiseRange -delta;
        var i, ind;
        var routeLength=0;
        for (i=firstIndex; i < n-1;i++){
            vertex=path.getAt(i+1);
            routeLength+=geoDistance(vertex1, vertex); //2 times faster than vertex.distanceFrom
            if (routeLength>=limit){
                ind=i;
                refuelStop=new RefuelStop(vertex,ind,routeLength);
                break;
            }
            vertex1=vertex;
        }
        return refuelStop;
    }
    /**
      * Tests, if a point lays within a circle around the
      * center of one of the refueling stops.
      * @param point point to be tested as GLatLng
      * @param radius the radius of the circle in kms.
      * @return the index>-1 of the refueling stop, the point is nearby
      * <br>index =-1: there is no refueling stop, the popint is nearby
      */
    function isPointNearRefuelStop(point, radius){
        var n=_refuelStops.length;
        var refuelStop;
        var center;
        var dist;
        var rv=-1;
        for (var i=0;i<n;i++){
            refuelStop=_refuelStops[i];
            center=refuelStop.center;
            dist=geoDistance(point,center);
            if (dist<=radius){
                rv=i;
                break;
            }
        }
        return rv;
    }
}

/**
 *Poi of a gas fillining station
 *@param marker a googlemaps.Marker object
 *@param name of the marker
 *@param desc description
 *@param detailUrl the url of the station details at www.gas-tankstellen.de
 *@param countryCode web country code, 2 characters lowercase
 *<br>Public fields
 *<br>all parameters of constructor
 *<br>distanceFromRoute distance from the route in kms
 *<br>distanceOnRoute distance on route from route start kms
 *<br> the last 2 fields will be added when stations are searched.
 */
function GasStationPoi(marker, name, desc,detailUrl,countryCode){
    this.marker=marker;
    this.name=name;
    this.desc=desc;
    this.stationDetailUrl=detailUrl;
    this.countryCode=countryCode;
    this.distanceFromRoute=null;
    this.distanceOnRoute=null;
    this.orderNumber=-1; //order in list of stations; is set in routeMain.outputPois()
    var resultLabels=new DivResultsTexts(); //english or german texts
   /**
     *Generates a Html string out of the descriptive fields.
     */
    this.toHtmlString=function(){
        var  content=this.titleToHtmlString() + this.bodyToHtmlString();
        return content;
    };
    /**
     *Generates the title line of the station info.
    */
    this.titleToHtmlString=function(){
        var flag;
        if (countryCode!=null){
            flag='<img width="13" height="10" alt="National flag" src="./flags/' + this.countryCode + '.png"/>';
        }else{
            flag="";
        }
        var titleLine=flag + " <b> " + this.orderNumber + ". " + this.name + "</b>";
        return titleLine;
    };
    /**
     *Generates the body of the station info.
    */
    this.bodyToHtmlString=function(){
        var latLng=this.marker.getPosition();
        var content="<br>" + this.desc +
        "<br>" + resultLabels.distOnRoute + this.distanceOnRoute + " km" +
        "<br>" + resultLabels.distFromRoute + this.distanceFromRoute + " km" +
        "<br>" + resultLabels.latLng + "&nbsp;" + latLng.toUrlValue(6);
        var detailUrl = this.stationDetailUrl;
        if (detailUrl !== null) {
            content += '<p><a href=' + detailUrl + ' target="_blank">Details</a></p>';
        }
        return content;
    };
}
/**
 * Class PoiAsKml used for generating POIs as a KML file
 */
function PoiAsKml(){
    /**
     *Gets the header tags of an xml file
     *@return the tags <?xml>, <kml>. <name>, <description> as a string
     */
    this.getHeadTags=function(){
        var txt=new Array();
        txt.push('&lt;?xml version="1.0" encoding="utf-8"?>');
        txt.push('<br>&lt;kml>&lt;Document>');
        txt.push('<br>&lt;name>routeCngs.kml&lt;/name>');
        txt.push('<br>&lt;description>CNG stations nearby the route&lt;/description>');
        return txt.join('&nbsp;');
    };
    /**
     * Gets the end tag of a KML file
     * @return closing tags </Document>, </kml> as a string
     */
    this.getEndTags=function(){
        return '<br>&lt;/Document> &lt;/kml>';
    };
    /**
     * Gets a POI element of a KML file
     * @param name name of the POI
     * @param description if the POI
     * @param latLng geograph. latituede and longitude of the POI
     * @return the tags of a single POI with the tags<Placemark>, <description>,
     * <Point>, <coordinates> and the closing tags as a string
     */
    this.getPoiElement=function(name, description, latLng){
        var txt=new Array();
        var lat=latLng.lat();
        var lng=latLng.lng();
        txt.push('<br>&lt;Placemark><br>&lt;name>&lt;![CDATA[' + name + ']]>&lt;/name>');
        txt.push('<br>&lt;description>&lt;![CDATA[' + description + ']]>&lt;/description>');
        txt.push('<br>&lt;Point>&lt;coordinates>');
        txt.push(lng + ',' + lat + ',0');
        txt.push('<br>&lt;/coordinates>&lt;/Point>');
        txt.push('<br>&lt;/Placemark>');
        return txt.join('&nbsp;');
    };
}
/**
 * Class PoiAsCsv used for generating POIs as a CSV file
 */
function PoiAsCsv(){
    /**
     * Gets the empty header tags for compatibility reasons with class PoiAsKml
     *@return empty string
     */
    this.getHeadTags=function(){
        return '';
    };
    /**
     * Gets the empty end tags for compatibility reasons with class PoiAsKml
     *@return empty string
     */
    this.getEndTags=function(){
        return '';
    };
    /**
     * Gets a POI element of a CSV file.
     * <br>A POI element is a line of the items longitude, latitude and name + description,
     * separated by commata ','
     * @param name name of the POI
     * @param description description of the POI
     * @param latLng geograph. latituede and longitude of the POI
     * @return string of longitude, latitude and name and desription, separated by ','
     */
    this.getPoiElement=function(name, description, latLng){
        var lat=latLng.lat();
        var lng=latLng.lng();
        //some browsers insert a linefeed for blank
        var s1=name.replace(/ /g, "&nbsp;");
        var s2=description.replace(/ /g, "&nbsp;");
        return '<br>"' + lng + '","' + lat + '","' + s1 + ';'+ s2 + '"';
    };
}
//end of classes
//Functions
/**
 *Gets the shortest distance of a point to the polyline.
 *@param poly polyline
 *@param latLng latLng of the point
 *@return {distance, routeIndex}
 *<br>distance: shortest distance of the point to the polyline in kms,
 *<br>routeOndex: the index of the polyline point with the shortest distance to the point
 */
function getProximity(poly, latLng){
    var path=poly.getPath();
    var length=path.getLength();
    var routeIndex=0;
    if (length > 1){
        var minDist = 40000;
        //var intermediateIndex = Math.round(length / 10);
        var dist;
        for (var n = 0 ; n < length; n++) {
            dist=geoDistance(latLng,path.getAt(n));
            if (dist < minDist) {
                minDist = dist;
                routeIndex=n;
            }
        }
    }
    return {
        'distance':minDist,
        'routeIndex':routeIndex
    };
}
/**
    * Get the geodetic distance of two points in km.
    * <br>The geodetic line is the shortest distance of 2 points on a sphere.
    * @param latlng1 geographical latitude, longitude of point 1 in degrees
    * @param latlng2 geographical latitude, longitude of point 2 in degrees
    * @return geodetic distance in km.
    * @see http://en.wikipedia.org/wiki/Great-circle_distance
    */
function geoDistance(latlng1,latlng2){
    var lat1=latlng1.lat()*toRadians;
    var lng1=latlng1.lng()*toRadians;
    var lat2=latlng2.lat()*toRadians;
    var lng2=latlng2.lng()*toRadians;
    var dlat=lat2 - lat1;
    var dlng=lng2 - lng1;
    var z=Math.sin(dlat*0.5);
    var x=z*z;
    z=Math.sin(dlng*0.5);
    x+=z*z*Math.cos(lat1)*Math.cos(lat2);
    var ds=2*Math.asin(Math.sqrt(x));
    return ds*earthRadius;
}






